@@ -1,7 +1,7 @@
 /*
    anubisusr.c
    
-   Copyright (C) 2004, 2005, 2007 The Anubis Team.
+   Copyright (C) 2004, 2005, 2007, 2008 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -17,23 +17,7 @@
    with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-#include "headers.h"
-#include "extern.h"
-#include "rcfile.h"
-#include <gsasl.h>
-#include "getopt.h"
-
-#define obstack_chunk_alloc malloc
-#define obstack_chunk_free free
-#include <obstack.h>
-
-#if defined(USE_GNUTLS) && defined(HAVE_GNUTLS_GNUTLS_H)
-# include <gnutls/gnutls.h>
-# define HAVE_TLS
-#endif /* USE_GNUTLS and HAVE_GNUTLS_GNUTLS_H */
+#include <anubisusr.h>
 
 #ifdef HAVE_TLS
 char *tls_cafile;
@@ -324,7 +308,8 @@ send_line (char *buf)
 int
 smtp_get_reply (struct smtp_reply *repl)
 {
-  char buf[LINEBUFFER + 1];
+  char *buf = NULL;
+  size_t bufsize = 0;
   char *p;
   int i;
   int cont = 0;
@@ -333,7 +318,7 @@ smtp_get_reply (struct smtp_reply *repl)
   do
     {
       size_t n;
-      int rc = stream_readline (iostream, buf, sizeof buf, &n);
+      int rc = stream_getline (iostream, &buf, &bufsize, &n);
 
       if (rc)
 	{
@@ -382,6 +367,7 @@ smtp_get_reply (struct smtp_reply *repl)
 	}
     }
   while (cont || *p == '-');
+  free (buf);
   obstack_1grow (&input_stk, 0);
   repl->base = obstack_finish (&input_stk);
 
@@ -519,7 +505,7 @@ parse_netrc (const char *filename)
   char *buf = NULL;
   size_t n = 0;
   int def_argc = 0;
-  char **def_argv;
+  char **def_argv = NULL;
   char **p_argv = NULL;
   int line = 0;
 
@@ -615,20 +601,6 @@ parse_netrc (const char *filename)
     }
 }
 
-/* FIXME: Add UTF-8 conversion */
-static int
-utf8cpy (char *dst, size_t * dstlen, char *src, size_t srclen)
-{
-  size_t len = strlen (src);
-
-  if (dst && *dstlen < len)
-    return GSASL_TOO_SMALL_BUFFER;
-  *dstlen = len;
-  if (dst)
-    strcpy (dst, src);
-  return GSASL_OK;
-}
-
 char *
 get_input (const char *prompt)
 {
@@ -645,159 +617,85 @@ get_input (const char *prompt)
   return buf;
 }
 
-int
-cb_anonymous (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
+static int
+callback (Gsasl *ctx, Gsasl_session *sctx, Gsasl_property prop)
 {
-  int rc;
+  int rc = GSASL_OK;
 
-  if (auth_args.anon_token == NULL)
-    auth_args.anon_token = get_input (_("Anonymous token: "));
-
-  if (auth_args.anon_token == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.anon_token,
-		strlen (auth_args.anon_token));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
-}
-
-int
-cb_authorization_id (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
-{
-  int rc;
-
-  if (auth_args.authorization_id == NULL)
-    auth_args.authorization_id = get_input (_("Authorization ID: "));
-
-  if (auth_args.authorization_id == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.authorization_id,
-		strlen (auth_args.authorization_id));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
-}
-
-int
-cb_authentication_id (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
-{
-  int rc;
-
-  if (auth_args.authentication_id == NULL)
-    auth_args.authentication_id = get_input (_("Authentication ID: "));
-
-  if (auth_args.authentication_id == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.authentication_id,
-		strlen (auth_args.authentication_id));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
-}
-
-int
-cb_password (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
-{
-  int rc;
-
-  if (auth_args.password == NULL)
-    auth_args.password = getpass (_("Password: "));
-
-  if (auth_args.password == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.password, strlen (auth_args.password));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
-}
-
-int
-cb_service (Gsasl_session_ctx * ctx, char *srv, size_t * srvlen,
-	    char *host, size_t * hostlen, char *srvname, size_t * srvnamelen)
-{
-  int rc;
-
-  if (auth_args.service == NULL)
-    auth_args.service = get_input (_("GSSAPI service name: "));
-
-  if (auth_args.hostname == NULL)
-    auth_args.hostname = get_input (_("Hostname of server: "));
-
-  if (srvnamelen && auth_args.service_name == NULL)
-    auth_args.service_name = get_input (_("Generic service name: "));
-
-  if (auth_args.service == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (auth_args.hostname == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  if (srvnamelen && auth_args.service_name == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (srv, srvlen, auth_args.service, strlen (auth_args.service));
-  if (rc != GSASL_OK)
-    return rc;
-
-  rc = utf8cpy (host, hostlen, auth_args.hostname,
-		strlen (auth_args.hostname));
-  if (rc != GSASL_OK)
-    return rc;
-
-  if (srvnamelen)
+  switch (prop)
     {
-      rc = utf8cpy (srvname, srvnamelen, auth_args.service_name,
-		    strlen (auth_args.service_name));
-      if (rc != GSASL_OK)
-	return rc;
+    case GSASL_PASSWORD:
+      if (auth_args.password == NULL)
+	auth_args.password = getpass (_("Password: "));
+
+      if (auth_args.password == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.password);
+      break;
+
+    case GSASL_SERVICE:
+      if (auth_args.service == NULL)
+	auth_args.service = get_input (_("GSSAPI service name: "));
+      if (auth_args.service == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.service);
+      break;
+
+    case GSASL_REALM:
+      if (auth_args.realm == NULL)
+	auth_args.realm = get_input (_("Client realm: "));
+      if (auth_args.realm == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.realm);
+      break;
+
+    case GSASL_HOSTNAME:
+      if (auth_args.hostname == NULL)
+	auth_args.hostname = get_input (_("Hostname of server: "));
+      if (auth_args.hostname == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.hostname);
+      break;
+
+    case GSASL_ANONYMOUS_TOKEN:
+      if (auth_args.anon_token == NULL)
+	auth_args.anon_token = get_input (_("Anonymous token: "));
+      if (auth_args.anon_token == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.anon_token);
+      break;
+
+    case GSASL_AUTHID:
+      if (auth_args.authentication_id == NULL)
+	auth_args.authentication_id = get_input (_("Authentication ID: "));
+      if (auth_args.authentication_id == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.authentication_id);
+      break;
+
+    case GSASL_AUTHZID:
+      if (auth_args.authorization_id == NULL)
+	auth_args.authorization_id = get_input (_("Authorization ID: "));
+      if (auth_args.authorization_id == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.authorization_id);
+      break;
+
+    case GSASL_PASSCODE:
+      if (auth_args.passcode == NULL)
+	auth_args.passcode = getpass (_("Passcode: "));
+      if (auth_args.passcode == NULL)
+	return GSASL_AUTHENTICATION_ERROR;
+      gsasl_property_set (sctx, prop, auth_args.passcode);
+      break;
+
+    default:
+      rc = GSASL_NO_CALLBACK;
+      error (_("Unsupported callback property %d"), prop);
+      break;
     }
 
-  return GSASL_OK;
-}
-
-int
-cb_passcode (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
-{
-  int rc;
-
-  if (auth_args.passcode == NULL)
-    auth_args.passcode = getpass (_("Passcode: "));
-
-  if (auth_args.passcode == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.passcode, strlen (auth_args.passcode));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
-}
-
-int
-cb_realm (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
-{
-  int rc;
-
-  if (auth_args.realm == NULL)
-    auth_args.realm = get_input (_("Client realm: "));
-
-  if (auth_args.realm == NULL)
-    return GSASL_AUTHENTICATION_ERROR;
-
-  rc = utf8cpy (out, outlen, auth_args.realm, strlen (auth_args.realm));
-  if (rc != GSASL_OK)
-    return rc;
-
-  return GSASL_OK;
+  return rc;
 }
 
 void
@@ -813,11 +711,11 @@ smtp_quit (void)
 /* GSASL Authentication */
 
 int
-do_gsasl_auth (Gsasl_ctx * ctx, char *mech)
+do_gsasl_auth (Gsasl *ctx, char *mech)
 {
   char *output;
   int rc;
-  Gsasl_session_ctx *sess_ctx = NULL;
+  Gsasl_session *sess_ctx = NULL;
   struct smtp_reply repl;
   char buf[LINEBUFFER + 1];
 
@@ -899,7 +797,7 @@ do_gsasl_auth (Gsasl_ctx * ctx, char *mech)
 void
 smtp_auth (void)
 {
-  Gsasl_ctx *ctx;
+  Gsasl *ctx;
   char *mech;
   int rc;
 
@@ -920,13 +818,7 @@ smtp_auth (void)
       exit (1);
     }
 
-  gsasl_client_callback_anonymous_set (ctx, cb_anonymous);
-  gsasl_client_callback_authentication_id_set (ctx, cb_authentication_id);
-  gsasl_client_callback_authorization_id_set (ctx, cb_authorization_id);
-  gsasl_client_callback_password_set (ctx, cb_password);
-  gsasl_client_callback_passcode_set (ctx, cb_passcode);
-  gsasl_client_callback_service_set (ctx, cb_service);
-  gsasl_client_callback_realm_set (ctx, cb_realm);
+  gsasl_callback_set (ctx, callback);
 
   do_gsasl_auth (ctx, mech);
 }
@@ -1152,47 +1044,6 @@ synch (void)
 }
 
 
-/* Main */
-#define OPT_VERSION          257
-#define OPT_HELP             258
-
-static struct option gnu_options[] = {
-  {"verbose", no_argument, 0, 'v'},
-  {"version", no_argument, 0, OPT_VERSION},
-  {"help", no_argument, 0, OPT_HELP},
-  {"file", required_argument, 0, 'f'},
-  {"netrc", required_argument, 0, 'n'},
-#ifdef HAVE_TLS
-  {"disable-tls", no_argument, 0, 'd'},
-  {"tls-cafile", required_argument, 0, 'C'},
-#endif
-  {"mechanism", required_argument, 0, 'm'},
-  {0, 0, 0, 0}
-};
-
-void
-help (void)
-{
-  puts (_("anubisusr -- Synchronize local and remote copies of the user's RC file"));
-  puts (_("Usage: anubisusr [OPTIONS] [URL]"));
-  puts (_("OPTIONS are:"));
-#ifdef HAVE_TLS
-  puts (_("  -d, --disable-tls       Disable TLS encryption"));
-  puts (_("  -C, --tls-cafile FILE   Use given CA file"));
-#endif
-  puts (_("  -f, --file FILE         Set user configuration file name"));
-  puts (_("  -m, --mechanism MECH    Restrict allowed SASL mechanisms"));
-  puts (_("  -n, --netrc FILE        Set .netrc file name"));
-  puts (_("  -v, --verbose           Verbose output. Multiple options\n"
-	  "                          increase the verbosity. Maximum is\n"
-	  "                          3"));
-  puts ("");
-  puts (_("  --version               Display program version"));
-  puts (_("  --help                  Display this help list"));
-  printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
-  exit (0);
-}
-
 #define NETRC_NAME ".netrc"
 void
 read_netrc (void)
@@ -1221,57 +1072,15 @@ xalloc_die ()
 int
 main (int argc, char **argv)
 {
-  int c;
-
+  int index;
+  
   progname = strrchr (argv[0], '/');
   if (!progname)
     progname = argv[0];
   else
     progname++;
 
-  while ((c = getopt_long (argc, argv, "dC:f:n:m:v", gnu_options, NULL)) != EOF)
-    {
-      switch (c)
-	{
-#ifdef HAVE_TLS
-	case 'd':
-	  enable_tls = 0;
-	  break;
-
-	case 'C':
-	  tls_cafile = optarg;
-	  break;
-
-#endif
-	case 'f':
-	  rcfile_name = optarg;
-	  break;
-
-	case 'n':
-	  netrc_name = optarg;
-	  break;
-	  
-	case 'm':
-	  add_mech (optarg);
-	  break;
-
-	case 'v':
-	  verbose++;
-	  break;
-
-	case OPT_VERSION:
-	  printf ("anubisusr (%s)\n", PACKAGE_STRING);
-	  exit (0);
-
-	case OPT_HELP:
-	  help ();
-	  break;
-
-	default:
-	  fprintf (stderr, "%s: unknown option -%c\n", progname, optopt);
-	  exit (1);
-	}
-    }
+  usr_get_options (argc, argv, &index);
 
   argc -= optind;
   argv += optind;
