@@ -2,7 +2,7 @@
    extern.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -113,4 +113,8 @@ extern char *outgoing_mail_rule;
 
 extern char *from_address;
 
+extern char *anubis_sasl_service;
+extern char *anubis_sasl_realm;
+extern char *anubis_sasl_hostname;
+
 /* EOF */
