@@ -2,7 +2,7 @@
    headers.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -31,6 +31,10 @@
   __attribute__ ((__format__ (__printf__, fmt, narg)))
 #endif
 
+#ifndef ANUBIS_NORETURN
+# define ANUBIS_NORETURN __attribute__((__noreturn__))
+#endif
+
 # if defined(HAVE_GNUTLS_GNUTLS_H)
 #  define HAVE_GNUTLS
 # endif	/* HAVE_GNUTLS_GNUTLS_H */
@@ -60,17 +64,17 @@
 
 #include <stdio.h>
 #ifdef STDC_HEADERS
-#include <stdlib.h>
-#include <stdarg.h>
+# include <stdlib.h>
+# include <stdarg.h>
 #endif /* STDC_HEADERS */
 #ifdef HAVE_STRING_H
-#include <string.h>
+# include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #ifdef HAVE_MEMORY_H
-#include <memory.h>
+# include <memory.h>
 #endif /* HAVE_MEMORY_H */
 #include <signal.h>
 #include <errno.h>
@@ -79,10 +83,10 @@
 #include <time.h>
 #include <pwd.h>
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+# include <sys/types.h>
 #endif /* HAVE_SYS_TYPES_H */
 #ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+# include <sys/stat.h>
 #endif /* HAVE_SYS_STAT_H */
 #include <sys/file.h>
 #include <sys/fcntl.h>
@@ -143,28 +147,7 @@
 #include <argcv.h>
 #include "list.h"
 
-#ifdef HAVE_SYSEXITS_H
-# include <sysexits.h>
-#else
-# define EX_OK          0       /* successful termination */
-# define EX__BASE       64      /* base value for error messages */
-# define EX_USAGE       64      /* command line usage error */
-# define EX_DATAERR     65      /* data format error */
-# define EX_NOINPUT     66      /* cannot open input */
-# define EX_NOUSER      67      /* addressee unknown */
-# define EX_NOHOST      68      /* host name unknown */
-# define EX_UNAVAILABLE 69      /* service unavailable */
-# define EX_SOFTWARE    70      /* internal software error */
-# define EX_OSERR       71      /* system error (e.g., can't fork) */
-# define EX_OSFILE      72      /* critical OS file missing */
-# define EX_CANTCREAT   73      /* can't create (user) output file */
-# define EX_IOERR       74      /* input/output error */
-# define EX_TEMPFAIL    75      /* temp failure; user is invited to retry */
-# define EX_PROTOCOL    76      /* remote error in protocol */
-# define EX_NOPERM      77      /* permission denied */
-# define EX_CONFIG      78      /* configuration error */
-# define EX__MAX        78      /* maximum listed value */
-#endif
+#include <sysexits.h>
 
 #ifndef INADDR_NONE
 # define INADDR_NONE (unsigned long)0xffffffff
@@ -321,10 +304,11 @@ int stream_set_write (struct net_stream *str, stream_write_t write);
 int stream_set_strerror (struct net_stream *str, stream_strerror_t strerr);
 int stream_close (NET_STREAM str);
 const char *stream_strerror (NET_STREAM str, int errcode);
-int stream_read (NET_STREAM str, char *buf, size_t size, size_t * nbytes);
+int stream_read (NET_STREAM str, char *buf, size_t size, size_t *nbytes);
 int stream_write (NET_STREAM str, const char *buf, size_t size,
-		  size_t * nbytes);
-int stream_readline (NET_STREAM str, char *buf, size_t size, size_t * nbytes);
+		  size_t *nbytes);
+int stream_readline (NET_STREAM str, char *buf, size_t size, size_t *nbytes);
+int stream_getline (NET_STREAM sd, char **vptr, size_t *maxlen, size_t *nread);
 int stream_destroy (NET_STREAM *);
 
 /* main.c */
@@ -349,11 +333,12 @@ void write_pid_file (void);
 
 /* errs.c */
 #define EXIT_ABORT 256
-void anubis_error (int, int, const char *, ...) ANUBIS_PRINTFLIKE(3,4);
+void anubis_error (int, int, const char *, ...)
+     ANUBIS_PRINTFLIKE(3,4);
 void anubis_warning (int error_code, const char *fmt, ...)
      ANUBIS_PRINTFLIKE(2,3);
 void socket_error (const char *);
-void hostname_error (char *);
+void hostname_error (const char *);
 
 /* log.c */
 void mprintf (const char *, ...);
@@ -585,6 +570,7 @@ void pgsql_db_init (void);
 
 /* transmode.c */
 int anubis_transparent_mode (struct sockaddr_in *addr);
+void session_prologue ();
 
 /* authmode.c */
 int anubis_authenticate_mode (struct sockaddr_in *addr);
@@ -598,7 +584,7 @@ void authmode_section_init (void);
 void auth_gsasl_init (void);
 int anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr);
 #ifdef WITH_GSASL
-void install_gsasl_stream (Gsasl_session_ctx * sess_ctx, NET_STREAM * stream);
+void install_gsasl_stream (Gsasl_session *sess_ctx, NET_STREAM * stream);
 #endif
 
 /* gsasl_srv.c */
