@@ -2,24 +2,20 @@
    ident.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -40,7 +36,7 @@
    returns 1 */
 
 static int
-ident_extract_username (char *reply, char *username, size_t size)
+ident_extract_username (char *reply, char **pusername)
 {
   char *p;
 
@@ -53,10 +49,10 @@ ident_extract_username (char *reply, char *username, size_t size)
   p = strchr (p, ':');
   if (!p)
     return 1;
-  p++;
-  if (strlen (p) >= size)
-    return 1;
-  strcpy (username, p);
+  do
+    p++;
+  while (*p == ' ');
+  assign_string (pusername, p);
   return 0;
 }
 
@@ -69,7 +65,7 @@ ident_extract_username (char *reply, char *username, size_t size)
    returns 1 */
 
 static int
-crypt_extract_username (char *reply, char *username, size_t size)
+crypt_extract_username (char *reply, char **pusername)
 {
   int i;
   char *p = reply;
@@ -83,18 +79,18 @@ crypt_extract_username (char *reply, char *username, size_t size)
 	return 1;
     }
 
-  if (strlen (p) >= size)
-    return 1;
-  strcpy (username, p);
+  assign_string (pusername, p);
   return 0;
 }
 
 int
-auth_ident (struct sockaddr_in *addr, char *user, int size)
+auth_ident (struct sockaddr_in *addr, char **user)
 {
   struct servent *sp;
   struct sockaddr_in ident;
-  char buf[LINEBUFFER + 1];
+  char *buf = NULL;
+  char inetd_buf[LINEBUFFER];
+  size_t size = 0;
   int sd = 0;
   int rc;
   NET_STREAM str;
@@ -123,10 +119,10 @@ auth_ident (struct sockaddr_in *addr, char *user, int size)
   info (VERBOSE, _("IDENT: connected to %s:%u"),
 	inet_ntoa (ident.sin_addr), ntohs (ident.sin_port));
 
-  snprintf (buf, LINEBUFFER,
+  snprintf (inetd_buf, sizeof inetd_buf,
 	    "%u , %u" CRLF, ntohs (addr->sin_port), session.anubis_port);
 
-  if ((rc = stream_write (str, buf, strlen (buf), &nbytes)))
+  if ((rc = stream_write (str, inetd_buf, strlen (inetd_buf), &nbytes)))
     {
       anubis_error (0, 0,
 		    _("IDENT: stream_write() failed: %s."),
@@ -134,7 +130,7 @@ auth_ident (struct sockaddr_in *addr, char *user, int size)
       net_close_stream (&str);
       return 0;
     }
-  if (recvline (CLIENT, str, buf, LINEBUFFER) == 0)
+  if (recvline (CLIENT, str, &buf, &size) == 0)
     {
       anubis_error (0, 0,
 		    _("IDENT: recvline() failed: %s."),
@@ -143,29 +139,30 @@ auth_ident (struct sockaddr_in *addr, char *user, int size)
       return 0;
     }
   net_close_stream (&str);
-  memset (user, 0, size);
 
   remcrlf (buf);
-  if (ident_extract_username (buf, user, size))
+  if (ident_extract_username (buf, user))
     {
       info (VERBOSE, _("IDENT: incorrect data."));
+      free (buf);
       return 0;
     }
+  free (buf);
 
   /******************************
    IDENTD DES decryption support
   *******************************/
 
-  if (strstr (user, "[") && strstr (user, "]"))
+  if (strstr (*user, "[") && strstr (*user, "]"))
     {
       int rs = 0;
       info (VERBOSE, _("IDENT: data probably encrypted with DES..."));
-      external_program (&rs, IDECRYPT_PATH, user, buf, LINEBUFFER);
+      external_program (&rs, IDECRYPT_PATH, *user, buf, LINEBUFFER);
       if (rs == -1)
 	return 0;
 
       remcrlf (buf);
-      if (crypt_extract_username (buf, user, size))
+      if (crypt_extract_username (buf, user))
 	{
 	  info (VERBOSE, _("IDENT: incorrect data (DES deciphered)."));
 	  return 0;
@@ -175,17 +172,17 @@ auth_ident (struct sockaddr_in *addr, char *user, int size)
 	  if (ntohl (ident.sin_addr.s_addr) == INADDR_LOOPBACK)
 	    {
 	      struct passwd *pwd;
-	      int uid = atoi (user);
+	      int uid = atoi (*user);
 	      pwd = getpwuid (uid);
 	      if (pwd != 0)
-		strncpy (user, (char *) pwd->pw_name, size - 1);
+		assign_string (user, pwd->pw_name);
 	      else
 		return 0;
 	    }
 	}
     }
 
-  info (VERBOSE, _("IDENT: resolved remote user to %s."), user);
+  info (VERBOSE, _("IDENT: resolved remote user to %s."), *user);
   return 1;			/* success */
 }
 
