@@ -2,24 +2,20 @@
    map.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -48,11 +44,10 @@ struct translate_env
   char *extaddr;
   char translate[65];
   char into[65];
-  int size;
 };
 
 void
-parse_transmap (int *cs, char *extuser, char *extaddr, char *dst, int size)
+parse_transmap (int *cs, char *extuser, char *extaddr, char **dst)
 {
   struct translate_env env;
 
@@ -60,7 +55,6 @@ parse_transmap (int *cs, char *extuser, char *extaddr, char *dst, int size)
   env.cs = -1;			/* failed by default: unmatched */
   env.extuser = extuser;
   env.extaddr = extaddr;
-  env.size = size;
 
   rcfile_process_section (CF_SUPERVISOR, "TRANSLATION", &env, NULL);
   *cs = env.cs;
@@ -70,8 +64,7 @@ parse_transmap (int *cs, char *extuser, char *extaddr, char *dst, int size)
 	{
 	  info (NORMAL, _("%s remapped to %s@localhost."),
 		env.translate, env.into);
-	  memset (dst, 0, size);
-	  strncpy (dst, env.into, size - 1);
+	  assign_string (dst, env.into);
 	}
       else
 	*cs = 0;		/* failed: invalid user name */
