@@ -1,23 +1,24 @@
-
-/* A Bison parser, made by GNU Bison 2.3a+.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-   
-   This program is free software: you can redistribute it and/or modify
+
+   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -28,7 +29,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -46,7 +47,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3a+"
+#define YYBISON_VERSION "2.3"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -59,9 +60,64 @@
 
 
 
-/* Copy the first part of user declarations.  */
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum rc_yytokentype {
+     EOL = 258,
+     T_BEGIN = 259,
+     T_END = 260,
+     AND = 261,
+     OR = 262,
+     NE = 263,
+     IF = 264,
+     FI = 265,
+     ELSE = 266,
+     RULE = 267,
+     DONE = 268,
+     CALL = 269,
+     STOP = 270,
+     ADD = 271,
+     REMOVE = 272,
+     MODIFY = 273,
+     IDENT = 274,
+     STRING = 275,
+     REGEX = 276,
+     D_BEGIN = 277,
+     T_MSGPART = 278,
+     NOT = 279
+   };
+#endif
+/* Tokens.  */
+#define EOL 258
+#define T_BEGIN 259
+#define T_END 260
+#define AND 261
+#define OR 262
+#define NE 263
+#define IF 264
+#define FI 265
+#define ELSE 266
+#define RULE 267
+#define DONE 268
+#define CALL 269
+#define STOP 270
+#define ADD 271
+#define REMOVE 272
+#define MODIFY 273
+#define IDENT 274
+#define STRING 275
+#define REGEX 276
+#define D_BEGIN 277
+#define T_MSGPART 278
+#define NOT 279
 
-/* Line 22 of yacc.c  */
+
+
+
+/* Copy the first part of user declarations.  */
 #line 1 "rcfile.y"
 
 /*
@@ -124,9 +180,6 @@ static int def_regex_modifier = R_POSIX;
 static struct rc_secdef *rc_secdef;
 
 
-/* Line 22 of yacc.c  */
-#line 129 "y.tab.c"
-
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 1
@@ -145,71 +198,10 @@ static struct rc_secdef *rc_secdef;
 # define YYTOKEN_TABLE 0
 #endif
 
-
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum rc_yytokentype {
-     EOL = 258,
-     T_BEGIN = 259,
-     T_END = 260,
-     AND = 261,
-     OR = 262,
-     NE = 263,
-     IF = 264,
-     FI = 265,
-     ELSE = 266,
-     RULE = 267,
-     DONE = 268,
-     CALL = 269,
-     STOP = 270,
-     ADD = 271,
-     REMOVE = 272,
-     MODIFY = 273,
-     IDENT = 274,
-     STRING = 275,
-     REGEX = 276,
-     D_BEGIN = 277,
-     T_MSGPART = 278,
-     NOT = 279
-   };
-#endif
-/* Tokens.  */
-#define EOL 258
-#define T_BEGIN 259
-#define T_END 260
-#define AND 261
-#define OR 262
-#define NE 263
-#define IF 264
-#define FI 265
-#define ELSE 266
-#define RULE 267
-#define DONE 268
-#define CALL 269
-#define STOP 270
-#define ADD 271
-#define REMOVE 272
-#define MODIFY 273
-#define IDENT 274
-#define STRING 275
-#define REGEX 276
-#define D_BEGIN 277
-#define T_MSGPART 278
-#define NOT 279
-
-
-
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-{
-
-/* Line 22 of yacc.c  */
 #line 62 "rcfile.y"
-
+{
   char *string;
   RC_SECTION *section;
   RC_STMT *stmt;
@@ -235,25 +227,22 @@ typedef union YYSTYPE
   } begin_sec;
   ANUBIS_LIST *list;
   int eq;
-
-
-
-/* Line 22 of yacc.c  */
-#line 243 "y.tab.c"
-} YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
+}
+/* Line 187 of yacc.c.  */
+#line 233 "y.tab.c"
+	YYSTYPE;
 # define rc_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
+# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 22 of yacc.c  */
-#line 257 "y.tab.c"
+/* Line 216 of yacc.c.  */
+#line 246 "y.tab.c"
 
 #ifdef short
 # undef short
@@ -328,14 +317,14 @@ typedef short int rc_yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int rc_yyi)
+YYID (int i)
 #else
 static int
-YYID (rc_yyi)
-    int rc_yyi;
+YYID (i)
+    int i;
 #endif
 {
-  return rc_yyi;
+  return i;
 }
 #endif
 
@@ -582,7 +571,7 @@ static const char *const rc_yytname[] =
   "NE", "IF", "FI", "ELSE", "RULE", "DONE", "CALL", "STOP", "ADD",
   "REMOVE", "MODIFY", "IDENT", "STRING", "REGEX", "D_BEGIN", "T_MSGPART",
   "NOT", "'('", "')'", "'='", "'['", "']'", "':'", "$accept", "input",
-  "seclist", "section", "begin", "$@1", "end", "stmtlist", "stmt",
+  "seclist", "section", "begin", "@1", "end", "stmtlist", "stmt",
   "asgn_stmt", "keyword", "arglist", "arg", "cond_stmt", "cond", "meq",
   "key", "opt_key", "msgpart", "opt_sep", "s_msgpart", "r_msgpart",
   "regex", "string_key", "expr", "modlist", "opt_modlist", "modifier",
@@ -934,20 +923,17 @@ rc_yy_symbol_print (rc_yyoutput, rc_yytype, rc_yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-rc_yy_stack_print (rc_yytype_int16 *rc_yybottom, rc_yytype_int16 *rc_yytop)
+rc_yy_stack_print (rc_yytype_int16 *bottom, rc_yytype_int16 *top)
 #else
 static void
-rc_yy_stack_print (rc_yybottom, rc_yytop)
-    rc_yytype_int16 *rc_yybottom;
-    rc_yytype_int16 *rc_yytop;
+rc_yy_stack_print (bottom, top)
+    rc_yytype_int16 *bottom;
+    rc_yytype_int16 *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; rc_yybottom <= rc_yytop; rc_yybottom++)
-    {
-      int rc_yybot = *rc_yybottom;
-      YYFPRINTF (stderr, " %d", rc_yybot);
-    }
+  for (; bottom <= top; ++bottom)
+    YYFPRINTF (stderr, " %d", *bottom);
   YYFPRINTF (stderr, "\n");
 }
 
@@ -981,11 +967,11 @@ rc_yy_reduce_print (rc_yyvsp, rc_yyrule)
   /* The symbols being reduced.  */
   for (rc_yyi = 0; rc_yyi < rc_yynrhs; rc_yyi++)
     {
-      YYFPRINTF (stderr, "   $%d = ", rc_yyi + 1);
+      fprintf (stderr, "   $%d = ", rc_yyi + 1);
       rc_yy_symbol_print (stderr, rc_yyrhs[rc_yyprhs[rc_yyrule] + rc_yyi],
 		       &(rc_yyvsp[(rc_yyi + 1) - (rc_yynrhs)])
 		       		       );
-      YYFPRINTF (stderr, "\n");
+      fprintf (stderr, "\n");
     }
 }
 
@@ -1285,10 +1271,10 @@ int rc_yyparse ();
 
 
 
-/* The lookahead symbol.  */
+/* The look-ahead symbol.  */
 int rc_yychar;
 
-/* The semantic value of the lookahead symbol.  */
+/* The semantic value of the look-ahead symbol.  */
 YYSTYPE rc_yylval;
 
 /* Number of syntax errors so far.  */
@@ -1328,7 +1314,7 @@ rc_yyparse ()
   int rc_yyresult;
   /* Number of tokens to shift before error messages enabled.  */
   int rc_yyerrstatus;
-  /* Lookahead token as an internal (translated) token number.  */
+  /* Look-ahead token as an internal (translated) token number.  */
   int rc_yytoken = 0;
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
@@ -1465,9 +1451,6 @@ rc_yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", rc_yystate));
 
-  if (rc_yystate == YYFINAL)
-    YYACCEPT;
-
   goto rc_yybackup;
 
 /*-----------.
@@ -1476,16 +1459,16 @@ rc_yyparse ()
 rc_yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     lookahead token if we need one and don't already have one.  */
+     look-ahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to lookahead token.  */
+  /* First try to decide what to do without reference to look-ahead token.  */
   rc_yyn = rc_yypact[rc_yystate];
   if (rc_yyn == YYPACT_NINF)
     goto rc_yydefault;
 
-  /* Not known => get a lookahead token if don't already have one.  */
+  /* Not known => get a look-ahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
   if (rc_yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1517,16 +1500,20 @@ rc_yybackup:
       goto rc_yyreduce;
     }
 
+  if (rc_yyn == YYFINAL)
+    YYACCEPT;
+
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (rc_yyerrstatus)
     rc_yyerrstatus--;
 
-  /* Shift the lookahead token.  */
+  /* Shift the look-ahead token.  */
   YY_SYMBOL_PRINT ("Shifting", rc_yytoken, &rc_yylval, &rc_yylloc);
 
-  /* Discard the shifted token.  */
-  rc_yychar = YYEMPTY;
+  /* Discard the shifted token unless it is eof.  */
+  if (rc_yychar != YYEOF)
+    rc_yychar = YYEMPTY;
 
   rc_yystate = rc_yyn;
   *++rc_yyvsp = rc_yylval;
@@ -1566,16 +1553,12 @@ rc_yyreduce:
   switch (rc_yyn)
     {
         case 2:
-
-/* Line 22 of yacc.c  */
 #line 115 "rcfile.y"
     {
            }
     break;
 
   case 3:
-
-/* Line 22 of yacc.c  */
 #line 120 "rcfile.y"
     {
 	     (rc_yyval.section) = rc_section = (rc_yyvsp[(1) - (1)].section);
@@ -1583,8 +1566,6 @@ rc_yyreduce:
     break;
 
   case 4:
-
-/* Line 22 of yacc.c  */
 #line 124 "rcfile.y"
     {
 	     if ((rc_yyvsp[(2) - (2)].section))
@@ -1603,8 +1584,6 @@ rc_yyreduce:
     break;
 
   case 5:
-
-/* Line 22 of yacc.c  */
 #line 139 "rcfile.y"
     {
 	     lex_clear_state ();
@@ -1618,8 +1597,6 @@ rc_yyreduce:
     break;
 
   case 6:
-
-/* Line 22 of yacc.c  */
 #line 151 "rcfile.y"
     {
 	     (rc_yyval.section) = NULL;
@@ -1627,8 +1604,6 @@ rc_yyreduce:
     break;
 
   case 7:
-
-/* Line 22 of yacc.c  */
 #line 155 "rcfile.y"
     {
 	     (rc_yyval.section) = rc_section_create ((rc_yyvsp[(1) - (3)].begin_sec).name, (rc_yyvsp[(1) - (3)].begin_sec).line, (rc_yyvsp[(2) - (3)].stmtlist).head);
@@ -1636,8 +1611,6 @@ rc_yyreduce:
     break;
 
   case 8:
-
-/* Line 22 of yacc.c  */
 #line 159 "rcfile.y"
     {
 	     (rc_yyval.section) = NULL;
@@ -1645,15 +1618,11 @@ rc_yyreduce:
     break;
 
   case 9:
-
-/* Line 22 of yacc.c  */
 #line 164 "rcfile.y"
     { verbatim (); }
     break;
 
   case 10:
-
-/* Line 22 of yacc.c  */
 #line 165 "rcfile.y"
     {
 	     (rc_yyval.begin_sec).line = cfg_line_num - 1;
@@ -1665,8 +1634,6 @@ rc_yyreduce:
     break;
 
   case 11:
-
-/* Line 22 of yacc.c  */
 #line 173 "rcfile.y"
     {
 	     (rc_yyval.begin_sec).line = cfg_line_num - 1;
@@ -1678,8 +1645,6 @@ rc_yyreduce:
     break;
 
   case 13:
-
-/* Line 22 of yacc.c  */
 #line 186 "rcfile.y"
     {
 	     (rc_yyval.stmtlist).head = (rc_yyval.stmtlist).tail = (rc_yyvsp[(1) - (1)].stmt);
@@ -1687,8 +1652,6 @@ rc_yyreduce:
     break;
 
   case 14:
-
-/* Line 22 of yacc.c  */
 #line 190 "rcfile.y"
     {
 	     if ((rc_yyvsp[(2) - (2)].stmt))
@@ -1707,8 +1670,6 @@ rc_yyreduce:
     break;
 
   case 15:
-
-/* Line 22 of yacc.c  */
 #line 207 "rcfile.y"
     {
 	     (rc_yyval.stmt) = NULL;
@@ -1716,8 +1677,6 @@ rc_yyreduce:
     break;
 
   case 21:
-
-/* Line 22 of yacc.c  */
 #line 216 "rcfile.y"
     {
 	     lex_clear_state ();
@@ -1728,8 +1687,6 @@ rc_yyreduce:
     break;
 
   case 22:
-
-/* Line 22 of yacc.c  */
 #line 225 "rcfile.y"
     {
 	     int flags;
@@ -1753,8 +1710,6 @@ rc_yyreduce:
     break;
 
   case 23:
-
-/* Line 22 of yacc.c  */
 #line 247 "rcfile.y"
     {
 	     verbatim ();
@@ -1762,8 +1717,6 @@ rc_yyreduce:
     break;
 
   case 24:
-
-/* Line 22 of yacc.c  */
 #line 253 "rcfile.y"
     {
 	     (rc_yyval.list) = list_create ();
@@ -1772,8 +1725,6 @@ rc_yyreduce:
     break;
 
   case 25:
-
-/* Line 22 of yacc.c  */
 #line 258 "rcfile.y"
     {
 	     list_append ((rc_yyvsp[(1) - (2)].list), (rc_yyvsp[(2) - (2)].string));
@@ -1782,8 +1733,6 @@ rc_yyreduce:
     break;
 
   case 27:
-
-/* Line 22 of yacc.c  */
 #line 268 "rcfile.y"
     {
 	     (rc_yyval.stmt) = rc_stmt_create (rc_stmt_cond);
@@ -1794,8 +1743,6 @@ rc_yyreduce:
     break;
 
   case 28:
-
-/* Line 22 of yacc.c  */
 #line 275 "rcfile.y"
     {
 	     (rc_yyval.stmt) = rc_stmt_create (rc_stmt_cond);
@@ -1806,8 +1753,6 @@ rc_yyreduce:
     break;
 
   case 30:
-
-/* Line 22 of yacc.c  */
 #line 285 "rcfile.y"
     {
 	     (rc_yyval.node) = (rc_yyvsp[(2) - (3)].node);
@@ -1815,8 +1760,6 @@ rc_yyreduce:
     break;
 
   case 31:
-
-/* Line 22 of yacc.c  */
 #line 289 "rcfile.y"
     {
 	     (rc_yyval.node) = rc_node_create (rc_node_bool);
@@ -1827,8 +1770,6 @@ rc_yyreduce:
     break;
 
   case 32:
-
-/* Line 22 of yacc.c  */
 #line 296 "rcfile.y"
     {
 	     (rc_yyval.node) = rc_node_create (rc_node_bool);
@@ -1839,8 +1780,6 @@ rc_yyreduce:
     break;
 
   case 33:
-
-/* Line 22 of yacc.c  */
 #line 303 "rcfile.y"
     {
 	     (rc_yyval.node) = rc_node_create (rc_node_bool);
@@ -1851,8 +1790,6 @@ rc_yyreduce:
     break;
 
   case 34:
-
-/* Line 22 of yacc.c  */
 #line 312 "rcfile.y"
     {
 	     (rc_yyval.eq) = 1;
@@ -1860,8 +1797,6 @@ rc_yyreduce:
     break;
 
   case 35:
-
-/* Line 22 of yacc.c  */
 #line 316 "rcfile.y"
     {
 	     (rc_yyval.eq) = 1;
@@ -1869,8 +1804,6 @@ rc_yyreduce:
     break;
 
   case 36:
-
-/* Line 22 of yacc.c  */
 #line 320 "rcfile.y"
     {
 	     (rc_yyval.eq) = 0;
@@ -1878,8 +1811,6 @@ rc_yyreduce:
     break;
 
   case 37:
-
-/* Line 22 of yacc.c  */
 #line 326 "rcfile.y"
     {
 	     (rc_yyval.msgpart).part = HEADER;
@@ -1889,8 +1820,6 @@ rc_yyreduce:
     break;
 
   case 38:
-
-/* Line 22 of yacc.c  */
 #line 332 "rcfile.y"
     {
 	     (rc_yyval.msgpart).part = HEADER;
@@ -1900,8 +1829,6 @@ rc_yyreduce:
     break;
 
   case 39:
-
-/* Line 22 of yacc.c  */
 #line 340 "rcfile.y"
     {
 	     (rc_yyval.msgpart).string = NULL;
@@ -1910,8 +1837,6 @@ rc_yyreduce:
     break;
 
   case 41:
-
-/* Line 22 of yacc.c  */
 #line 348 "rcfile.y"
     {
 	     (rc_yyval.msgpart) = (rc_yyvsp[(2) - (2)].msgpart);
@@ -1921,8 +1846,6 @@ rc_yyreduce:
     break;
 
   case 43:
-
-/* Line 22 of yacc.c  */
 #line 357 "rcfile.y"
     {
 	     (rc_yyval.string) = NULL;
@@ -1930,8 +1853,6 @@ rc_yyreduce:
     break;
 
   case 44:
-
-/* Line 22 of yacc.c  */
 #line 361 "rcfile.y"
     {
 	     (rc_yyval.string) = (rc_yyvsp[(2) - (3)].string);
@@ -1939,8 +1860,6 @@ rc_yyreduce:
     break;
 
   case 45:
-
-/* Line 22 of yacc.c  */
 #line 367 "rcfile.y"
     {
 	     (rc_yyval.msgpart) = (rc_yyvsp[(1) - (1)].msgpart);
@@ -1950,8 +1869,6 @@ rc_yyreduce:
     break;
 
   case 46:
-
-/* Line 22 of yacc.c  */
 #line 375 "rcfile.y"
     {
 	     (rc_yyval.msgpart) = (rc_yyvsp[(1) - (1)].msgpart);
@@ -1964,8 +1881,6 @@ rc_yyreduce:
     break;
 
   case 47:
-
-/* Line 22 of yacc.c  */
 #line 386 "rcfile.y"
     {
 	     (rc_yyval.regex) = anubis_regex_compile ((rc_yyvsp[(3) - (4)].string), (rc_yyvsp[(1) - (4)].num));
@@ -1979,8 +1894,6 @@ rc_yyreduce:
     break;
 
   case 48:
-
-/* Line 22 of yacc.c  */
 #line 398 "rcfile.y"
     {
 	     (rc_yyval.string) = NULL;
@@ -1988,8 +1901,6 @@ rc_yyreduce:
     break;
 
   case 49:
-
-/* Line 22 of yacc.c  */
 #line 402 "rcfile.y"
     {
 	     (rc_yyval.string) = (rc_yyvsp[(2) - (3)].string);
@@ -1997,8 +1908,6 @@ rc_yyreduce:
     break;
 
   case 50:
-
-/* Line 22 of yacc.c  */
 #line 408 "rcfile.y"
     {
 	     RC_NODE *node = rc_node_create (rc_node_expr);
@@ -2020,8 +1929,6 @@ rc_yyreduce:
     break;
 
   case 51:
-
-/* Line 22 of yacc.c  */
 #line 428 "rcfile.y"
     {
 	     (rc_yyval.num) = def_regex_modifier;
@@ -2031,8 +1938,6 @@ rc_yyreduce:
     break;
 
   case 52:
-
-/* Line 22 of yacc.c  */
 #line 434 "rcfile.y"
     {
 	     reg_modifier_add (&(rc_yyvsp[(1) - (2)].num), (rc_yyvsp[(2) - (2)].string));
@@ -2042,8 +1947,6 @@ rc_yyreduce:
     break;
 
   case 53:
-
-/* Line 22 of yacc.c  */
 #line 442 "rcfile.y"
     {
 	     (rc_yyval.num) = def_regex_modifier;
@@ -2051,8 +1954,6 @@ rc_yyreduce:
     break;
 
   case 55:
-
-/* Line 22 of yacc.c  */
 #line 449 "rcfile.y"
     {
 	     (rc_yyval.string) = (rc_yyvsp[(2) - (2)].string);
@@ -2060,8 +1961,6 @@ rc_yyreduce:
     break;
 
   case 56:
-
-/* Line 22 of yacc.c  */
 #line 455 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2070,8 +1969,6 @@ rc_yyreduce:
     break;
 
   case 59:
-
-/* Line 22 of yacc.c  */
 #line 468 "rcfile.y"
     {
 	     (rc_yyval.stmt) = rc_stmt_create (rc_stmt_rule);
@@ -2081,8 +1978,6 @@ rc_yyreduce:
     break;
 
   case 60:
-
-/* Line 22 of yacc.c  */
 #line 476 "rcfile.y"
     {
 	     (rc_yyval.node) = rc_node_create (rc_node_expr);
@@ -2094,8 +1989,6 @@ rc_yyreduce:
     break;
 
   case 61:
-
-/* Line 22 of yacc.c  */
 #line 486 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2104,8 +1997,6 @@ rc_yyreduce:
     break;
 
   case 64:
-
-/* Line 22 of yacc.c  */
 #line 497 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2121,8 +2012,6 @@ rc_yyreduce:
     break;
 
   case 65:
-
-/* Line 22 of yacc.c  */
 #line 509 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2138,8 +2027,6 @@ rc_yyreduce:
     break;
 
   case 66:
-
-/* Line 22 of yacc.c  */
 #line 521 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2155,8 +2042,6 @@ rc_yyreduce:
     break;
 
   case 67:
-
-/* Line 22 of yacc.c  */
 #line 533 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2172,8 +2057,6 @@ rc_yyreduce:
     break;
 
   case 68:
-
-/* Line 22 of yacc.c  */
 #line 545 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2189,8 +2072,6 @@ rc_yyreduce:
     break;
 
   case 69:
-
-/* Line 22 of yacc.c  */
 #line 557 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2210,8 +2091,6 @@ rc_yyreduce:
     break;
 
   case 70:
-
-/* Line 22 of yacc.c  */
 #line 575 "rcfile.y"
     {
 	     if (!is_prog_allowed ())
@@ -2223,9 +2102,8 @@ rc_yyreduce:
     break;
 
 
-
-/* Line 22 of yacc.c  */
-#line 2229 "y.tab.c"
+/* Line 1267 of yacc.c.  */
+#line 2107 "y.tab.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", rc_yyr1[rc_yyn], &rc_yyval, &rc_yyloc);
@@ -2301,7 +2179,7 @@ rc_yyerrlab:
 
   if (rc_yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse lookahead token after an
+      /* If just tried and failed to reuse look-ahead token after an
 	 error, discard it.  */
 
       if (rc_yychar <= YYEOF)
@@ -2318,7 +2196,7 @@ rc_yyerrlab:
 	}
     }
 
-  /* Else will try to reuse lookahead token after shifting the error
+  /* Else will try to reuse look-ahead token after shifting the error
      token.  */
   goto rc_yyerrlab1;
 
@@ -2375,6 +2253,9 @@ rc_yyerrlab1:
       YY_STACK_PRINT (rc_yyss, rc_yyssp);
     }
 
+  if (rc_yyn == YYFINAL)
+    YYACCEPT;
+
   *++rc_yyvsp = rc_yylval;
 
 
@@ -2399,7 +2280,7 @@ rc_yyabortlab:
   rc_yyresult = 1;
   goto rc_yyreturn;
 
-#if !defined(rc_yyoverflow) || YYERROR_VERBOSE
+#ifndef rc_yyoverflow
 /*-------------------------------------------------.
 | rc_yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2410,7 +2291,7 @@ rc_yyexhaustedlab:
 #endif
 
 rc_yyreturn:
-  if (rc_yychar != YYEMPTY)
+  if (rc_yychar != YYEOF && rc_yychar != YYEMPTY)
      rc_yydestruct ("Cleanup: discarding lookahead",
 		 rc_yytoken, &rc_yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2436,8 +2317,6 @@ rc_yyreturn:
 }
 
 
-
-/* Line 22 of yacc.c  */
 #line 584 "rcfile.y"
 
 
