@@ -2,24 +2,20 @@
    rcfile.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -256,6 +252,10 @@ process_rcfile (int method)
 #define KW_ESMTP_REALM              27
 #define KW_ESMTP_ALLOWED_MECH       28
 #define KW_ESMTP_REQUIRE_ENCRYPTION 29
+#define KW_INCOMING_MAIL_RULE       30
+#define KW_OUTGOING_MAIL_RULE       31
+#define KW_HANG                     32
+#define KW_ALLOW_HANG               33
 
 char **
 list_to_argv (ANUBIS_LIST * list)
@@ -274,6 +274,15 @@ list_to_argv (ANUBIS_LIST * list)
   return argv;
 }
 
+/* When HANG=NUMBER is set in CONTROL section, `_anubis_hang' is set and
+   Anubis will sleep for one second intervals, decrementing `_anubis_hang'
+   until it's zero.  Thus you can force the program to continue by attaching
+   a debugger and setting it to 0 yourself.  */
+static volatile unsigned long _anubis_hang;
+
+/* List of users who are allowed to use HANG in their profiles */
+ANUBIS_LIST *allow_hang_users; 
+
 int
 control_parser (int method, int key, ANUBIS_LIST * arglist,
 		void *inv_data, void *func_data, MESSAGE * msg)
@@ -282,8 +291,8 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
 
   switch (key) {
   case KW_BIND:
-    parse_mtahost (arg, session.anubis, &session.anubis_port);
-    if (strlen (session.anubis) != 0)
+    parse_mtahost (arg, &session.anubis, &session.anubis_port);
+    if (session.anubis && strlen (session.anubis) != 0)
       topt |= T_NAMES;
     break;
 
@@ -327,14 +336,15 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
     break;
 
   case KW_USER_NOTPRIVILEGED:
-    safe_strcpy (session.notprivileged, arg);
+    assign_string (&session.notprivileged, arg);
+    topt |= T_USER_NOTPRIVIL;
     break;
 
   case KW_LOGFILE:
     if (method == CF_CLIENT)
       {
 	xfree (options.ulogfile);
-	options.ulogfile = allocbuf (arg, MAXPATHLEN);
+	options.ulogfile = xstrdup (arg);
       }
     else if (getpid () == 0)
       anubis_warning (0,
@@ -343,7 +353,7 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
       {
 	topt |= T_DISABLE_SYSLOG;
 	xfree (options.ulogfile);
-	options.ulogfile = allocbuf (arg, MAXPATHLEN);
+	options.ulogfile = xstrdup (arg);
       }
     break;
 
@@ -380,7 +390,7 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
 	      }
 	    else
 	      {
-		options.tracefile = allocbuf (arg, MAXPATHLEN);
+		options.tracefile = xstrdup (arg);
 		topt |= T_TRACEFILE_USR;
 	      }
 	  }
@@ -388,15 +398,20 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
     break;
 
   case KW_REMOTE_MTA:
-    parse_mtaport (arg, session.mta, &session.mta_port);
+    parse_mtaport (arg, &session.mta, &session.mta_port);
     break;
 
   case KW_LOCAL_MTA:
-    xfree (session.execpath);
-    xfree_pptr (session.execargs);
-    session.execpath = strdup (arg);
-    session.execargs = list_to_argv (arglist);
-    topt |= T_LOCAL_MTA;
+    if (!(topt & T_LOCAL_MTA)) /* Command line option overrides config */
+                               /* FIXME: generally speaking *all* command
+				  line options should */
+      { 
+	xfree (session.execpath);
+	argcv_free (-1, session.execargs);
+	session.execpath = strdup (arg);
+	session.execargs = list_to_argv (arglist);
+	topt |= T_LOCAL_MTA;
+      }
     break;
 
 #if defined (WITH_GSASL)
@@ -405,8 +420,8 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
       char *p = strchr (arg, ':');
       if (p)
 	{
-	  auth_password = strdup (++p);
-	  *--p = '\0';
+	  *p++ = 0;
+	  auth_password = strdup (p);
 	  authentication_id = strdup (arg);
 	  authorization_id = strdup (arg);
 	  topt |= T_ESMTP_AUTH;
@@ -476,7 +491,7 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
 
 #ifdef USE_SOCKS_PROXY
   case KW_SOCKS_PROXY:
-    parse_mtaport (arg, session.socks, &session.socks_port);
+    parse_mtaport (arg, &session.socks, &session.socks_port);
     if_empty_set (session.socks, topt, T_SOCKS);
     break;
 
@@ -490,9 +505,9 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
       p = strchr (arg, ':');
       if (p)
 	{
-	  safe_strcpy (session.socks_password, ++p);
-	  *--p = '\0';
-	  safe_strcpy (session.socks_username, arg);
+	  *p++ = 0;
+	  assign_string (&session.socks_password, p);
+	  assign_string (session.socks_username, arg);
 	  topt |= T_SOCKS_AUTH;
 	}
       break;
@@ -509,13 +524,58 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
 
 #ifdef WITH_GSASL
   case KW_MODE:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    if (anubis_set_mode (arg))
-      return RC_KW_ERROR;
+    if (anubis_mode != anubis_mda) /* Special case. See comment to
+				      KW_LOCAL_MAILER directive, though */
+      {
+	if (list_count (arglist) != 1)
+	  return RC_KW_ERROR;
+	if (anubis_set_mode (arg))
+	  return RC_KW_ERROR;
+      }
     break;
 #endif /* WITH_GSASL */
 
+  case KW_INCOMING_MAIL_RULE:
+    incoming_mail_rule = strdup (arg);
+    break;
+    
+  case KW_OUTGOING_MAIL_RULE:
+    outgoing_mail_rule = strdup (arg);
+    break;
+
+  case KW_ALLOW_HANG:
+    {
+      char *p;
+      ITERATOR *itr = iterator_create (arglist);
+      
+      allow_hang_users = list_create ();
+      for (p = iterator_first (itr); p; p = iterator_next (itr))
+	list_append (allow_hang_users, strdup (p));
+    }
+    break;
+    
+  case KW_HANG:
+    if (list_locate (allow_hang_users, session.clientname, anubis_name_cmp))
+      {
+	int keep_termlevel = options.termlevel;
+
+	_anubis_hang = atoi (arg ? arg : "3600");
+	options.termlevel = DEBUG;
+	anubis_warning (0, ngettext ("Child process suspended for %lu second",
+				     "Child process suspended for %lu seconds",
+				     _anubis_hang),
+			_anubis_hang);
+	options.termlevel = keep_termlevel;
+	
+	while (_anubis_hang-- > 0)
+	  sleep (1);
+      }
+    else
+      anubis_warning (0,
+		      _("Command HANG is not allowed for user `%s'"),
+		      session.clientname);
+    break;
+    
   default:
     return RC_KW_UNKNOWN;
   }
@@ -526,6 +586,9 @@ static struct rc_kwdef init_kw[] = {
   { "bind",         KW_BIND },
   { "local-domain", KW_LOCAL_DOMAIN },
   { "mode",         KW_MODE },
+  { "incoming-mail-rule", KW_INCOMING_MAIL_RULE },
+  { "outgoing-mail-rule", KW_OUTGOING_MAIL_RULE },
+  { "ALLOW-HANG",   KW_ALLOW_HANG },
   { NULL },
 };
 
@@ -560,6 +623,7 @@ static struct rc_secdef_child init_supervisor_sect_child = {
 struct rc_kwdef client_kw[] = {
   { "logfile",  KW_LOGFILE },
   { "loglevel", KW_LOGLEVEL },
+  { "HANG",     KW_HANG },
   { NULL },
 };
 
@@ -628,19 +692,19 @@ tls_parser (int method, int key, ANUBIS_LIST * arglist,
 
   case KW_SSL_CERT:
     xfree (secure.cert);
-    secure.cert = allocbuf (arg, MAXPATHLEN);
+    secure.cert = xstrdup (arg);
     break;
 
   case KW_SSL_KEY:
     xfree (secure.key);
-    secure.key = allocbuf (arg, MAXPATHLEN);
+    secure.key = xstrdup (arg);
     if (method == CF_CLIENT)
       topt |= T_SSL_CKCLIENT;
     break;
 
   case KW_SSL_CAFILE:
     xfree (secure.cafile);
-    secure.cafile = allocbuf (arg, MAXPATHLEN);
+    secure.cafile = xstrdup (arg);
     break;
 
   default:
@@ -719,7 +783,7 @@ rule_parser (int method, int key, ANUBIS_LIST * arglist,
   case KW_EXTERNAL_BODY_PROCESSOR:
     argv = list_to_argv (arglist);
     message_external_proc (msg, argv);
-    xfree_pptr (argv);
+    argcv_free (-1, argv);
     break;
 
   default:
@@ -794,7 +858,9 @@ char *
 user_rcfile_name (void)
 {
   if (session.rcfile_name)
-    return strdup (session.rcfile_name);
+    {
+      return strdup (session.rcfile_name);
+    }
   else
     {
       char homedir[MAXPATHLEN + 1];
