@@ -2,7 +2,7 @@
    tunnel.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -220,11 +220,11 @@ collect_body (MESSAGE * msg)
   while (state != ST_DONE
 	 && (nread = recvline (SERVER, remote_client, &buf, &size)))
     {
-      if (strncmp (buf, "." CRLF, 3) == 0)	/* EOM */
-	break;
-
       remcrlf (buf);
 
+      if (strcmp (buf, ".") == 0)	/* EOM */
+	break;
+
       if (msg->boundary)
 	{
 	  switch (state)
@@ -312,7 +312,7 @@ smtp_session_transparent (void)
      First of all, transfer a welcome message.
    */
 
-  info (VERBOSE, _("Transferring message(s)..."));
+  info (VERBOSE, _("Transferring messages..."));
   get_response_smtp (CLIENT, remote_server, &command, &size);
 
   if (command &&
@@ -351,6 +351,8 @@ smtp_session_transparent (void)
   message_init (&msg);
   while (recvline (SERVER, remote_client, &command, &size))
     {
+      remcrlf (command);
+
       if (process_command (&msg, command))
 	continue;
 
@@ -390,11 +392,13 @@ smtp_session (void)
 
   info (VERBOSE, _("Starting SMTP session..."));
   smtp_begin ();
-  info (VERBOSE, _("Transferring message(s)..."));
+  info (VERBOSE, _("Transferring messages..."));
 
   message_init (&msg);
   while (recvline (SERVER, remote_client, &command, &size))
     {
+      remcrlf (command);
+      
       if (process_command (&msg, command))
 	continue;
 
@@ -557,7 +561,6 @@ process_command (MESSAGE * msg, char *command)
   char buf[LINEBUFFER + 1];
 
   safe_strcpy (buf, command);	/* make a back-up */
-  remcrlf (buf);
   make_lowercase (buf);
   save_command (msg, buf);
 
@@ -703,6 +706,35 @@ handle_ehlo (char *command, char **reply)
   return 0;
 }
 
+#define CMD_MAIL_FROM "mail from:"
+#define CMD_RCPT_TO "rcpt to:"
+
+static void
+strip_inter_ws(char *buf, size_t len, size_t pfxlen)
+{
+  if (isspace (buf[pfxlen]))
+    {
+      size_t i;
+      for (i = pfxlen; i < len && isspace (buf[i]); i++)
+	;
+      memmove (buf + pfxlen, buf + i, len - i + 1);
+    }
+}
+
+static int
+is_prefix(char *buf, char *pfx)
+{
+  while (*pfx)
+    {
+      int b = *buf++;
+      int c = *pfx++;
+      
+      if (!b || toupper (c) != toupper (b))
+	return 0;
+    }
+  return 1;
+}
+
 static int
 transfer_command (MESSAGE * msg, char *command)
 {
@@ -710,11 +742,21 @@ transfer_command (MESSAGE * msg, char *command)
   size_t reply_size = 0;
   char *buf = NULL;
   int rc = 1; /* OK */
+  size_t len;
   
   assign_string (&buf, command);
   make_lowercase (buf);
+
+  len = strlen (command);
+  if (len > sizeof (CMD_MAIL_FROM)
+      && is_prefix (command, CMD_MAIL_FROM))
+    strip_inter_ws (command, len, sizeof (CMD_MAIL_FROM) - 1);
+  else if (len > sizeof (CMD_RCPT_TO)
+	   && is_prefix (command, CMD_RCPT_TO))
+    strip_inter_ws (command, len, sizeof (CMD_RCPT_TO) - 1);
   
   swrite (CLIENT, remote_server, command);
+  swrite (CLIENT, remote_server, CRLF);
 
   if (!strncmp (buf, "ehlo", 4))
     {
@@ -765,7 +807,12 @@ process_data (MESSAGE * msg)
   transfer_body (msg);
 
   if (recvline (CLIENT, remote_server, &buf, &size))
-    swrite (SERVER, remote_client, buf);
+    {
+      remcrlf (buf);
+      
+      swrite (SERVER, remote_client, buf);
+      send_eol (SERVER, remote_client);
+    }
   free (buf);
 
   message_free (msg);
@@ -791,9 +838,11 @@ raw_transfer (void)
   char *buf = NULL;
   while (recvline (SERVER, remote_client, &buf, &size) > 0)
     {
-      if (strncmp (buf, "." CRLF, 3) == 0)	/* EOM */
+      remcrlf (buf);
+      if (strcmp (buf, ".") == 0)	/* EOM */
 	break;
       swrite (CLIENT, remote_server, buf);
+      send_eol (CLIENT, remote_server);
     }
   free (buf);
 }
