@@ -2,24 +2,20 @@
    xdatabase.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2004 The Anubis Team.
+   Copyright (C) 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -38,23 +34,24 @@ xdatabase_enable ()
 }
 
 void
-xdatabase_capability (char *reply, size_t reply_size)
+xdatabase_capability (char **preply, size_t *preply_size)
 {
   static char capa_string[] = "250-XDATABASE\r\n";
   size_t capa_len = strlen (capa_string);
   size_t len;
   char *p;
-
+  char *reply;
+  
   if (!xdatabase_active)
     return;
 
-  if (strlen (reply) + capa_len >= reply_size)
+  if (strlen (*preply) + capa_len >= *preply_size)
     {
-      anubis_error (0, 0,
-		    _("Cannot add capability: not enough buffer space."));
-      return;
+      *preply_size = strlen (*preply) + capa_len + 1;
+      *preply = xrealloc (*preply, *preply_size);
     }
-
+  reply = *preply;
+  
   p = strstr (reply, "250 ");
   if (!p)
     {
@@ -92,11 +89,11 @@ make_temp_file (struct obstack *stk, char *rcname, char **name)
 
   save_umask = umask (077);
   fp = fopen (p, "w");
-  umask (save_umask);
   if (!fp)
-    {
-      anubis_error (0, errno, _("Cannot open temporary file %s: %s"), p);
-    }
+    anubis_error (0, errno, _("Cannot open temporary file %s: %s"),
+		  p, strerror (errno));
+
+  umask (save_umask);
   return fp;
 }
 
@@ -122,10 +119,10 @@ _xdb_error_printer (void *data,
 static void
 xupload ()
 {
-  int n;
   char *tempname;
   FILE *tempfile;
-  char line[LINEBUFFER + 1];
+  char *line = NULL;
+  size_t size = 0;
   RC_SECTION *sec;
   struct obstack stk;
   char *rcname;
@@ -146,7 +143,7 @@ xupload ()
   swrite (SERVER, remote_client,
 	  "354 Enter configuration settings, end with \".\" on a line by itself\r\n");
 
-  while ((n = recvline (SERVER, remote_client, line, sizeof (line) - 1)) > 0)
+  while (recvline (SERVER, remote_client, &line, &size) > 0)
     {
       remcrlf (line);
       if (strcmp (line, ".") == 0)	/* EOM */
@@ -154,6 +151,7 @@ xupload ()
       fputs (line, tempfile);
       fputc ('\n', tempfile);
     }
+  free (line);  
 
   fclose (tempfile);
 
